//
// XPFiremon
//
// Monitors the status of the XP firewall as well as allowing control from the system tray.  Requires
// administrative access to manage the firewall.
//
// 1.0 - Works as admin to monitor and spawn management
//
// Future Plans:
// - verify it can run as a regular user, if not try to make it
// - Add settings modification warning for new ports or applications allowed
// - Maybe one day add in outbound alerting(won't stop it, just alert to new programs detected going out)
//
// Thanks to Microsoft for all the nice Firewall API wrapper code
//
// Feel free to use this code as you like or just verify it's a clean program.  All I ask is that if
// you add something pass the changes back for the rest of us.
//
// By Inferno (inferno@shadowlair.com)
//
#include <windows.h>
#include <crtdbg.h>
#include <objbase.h>
#include <netfw.h>
#include <oleauto.h>
#include <stdio.h>
#include "resource.h"

LPCSTR szClassName = "XPFiremon";/* Class name for main hidden window */
HBITMAP gSystrayBMP;/* Off-Screen bitmap to render SysTray onto */
HDC gSystrayHDC;/* An HDC used to draw to the off-screen bitmap */
HANDLE gHInstance;/* Instance handle for this application */
HWND gHwnd;/* The main hidden window that receives all events */
BOOL gAlready = FALSE;/* Flag to track if the SysTray has already been added */
int gCount =0;/* The demo counter displayed on the SysTray */

// Variables to go between the threads
static BOOL bActive = FALSE;
static BOOL bServiceActive = FALSE;
static INetFwProfile * fwProfileStatic = NULL;

// HANDLE to our window for changing systray icon
HINSTANCE hInst = NULL;

#define PRINTF ////

HRESULT WindowsFirewallInitialize(OUT INetFwProfile ** fwProfile)
{
    HRESULT hr = S_OK;
    INetFwMgr* fwMgr = NULL;
    INetFwPolicy* fwPolicy = NULL;
    _ASSERT(fwProfile != NULL);

    *fwProfile = NULL;

    // Create an instance of the firewall settings manager.
    hr = CoCreateInstance(
            __uuidof(NetFwMgr),
            NULL,
            CLSCTX_INPROC_SERVER,
            __uuidof(INetFwMgr),
            (void**)&fwMgr
            );
    if (FAILED(hr))
    {
        PRINTF("CoCreateInstance failed: 0x%08lx\n", hr);
        goto error;
    }

    // Retrieve the local firewall policy.
    hr = fwMgr->get_LocalPolicy(&fwPolicy);
    if (FAILED(hr))
    {
        PRINTF("get_LocalPolicy failed: 0x%08lx\n", hr);
        goto error;
    }

    // Retrieve the firewall profile currently in effect.
    hr = fwPolicy->get_CurrentProfile(fwProfile);
    if (FAILED(hr))
    {
        PRINTF("get_CurrentProfile failed: 0x%08lx\n", hr);
        goto error;
    }
    
    // Copy the fwProfile to the static buffer for use in our other thread
    fwProfileStatic = (INetFwProfile *)malloc(sizeof(fwProfile));
    if(!fwProfileStatic)
       goto error;
    memcpy(&fwProfileStatic, fwProfile, sizeof(fwProfileStatic));

error:

    // Release the local firewall policy.
    if (fwPolicy != NULL)
    {
        fwPolicy->Release();
    }

    // Release the firewall settings manager.
    if (fwMgr != NULL)
    {
        fwMgr->Release();
    }

    return hr;
}


void WindowsFirewallCleanup(IN INetFwProfile* fwProfile)
{
    // Release the firewall profile.
    if (fwProfile != NULL)
    {
        fwProfile->Release();
    }
}


HRESULT WindowsFirewallIsOn(IN INetFwProfile* fwProfile, OUT BOOL* fwOn)
{
    HRESULT hr = S_OK;
    VARIANT_BOOL fwEnabled;

    _ASSERT(fwProfile != NULL);
    _ASSERT(fwOn != NULL);

    *fwOn = FALSE;

    // Get the current state of the firewall.
    hr = fwProfile->get_FirewallEnabled(&fwEnabled);
    if (FAILED(hr))
    {
        PRINTF("get_FirewallEnabled failed: 0x%08lx\n", hr);
        goto error;
    }

    // Check to see if the firewall is on.
    if (fwEnabled != VARIANT_FALSE)
    {
        *fwOn = TRUE;
        //MessageBox(NULL, "The firewall is on.\n", "Test", NULL);
    }
    else
    {
        PRINTF("The firewall is off.\n");
    }

    return hr;

error:

    bActive = FALSE;
    return hr;
}


HRESULT WindowsFirewallTurnOn(IN INetFwProfile* fwProfile)
{
    HRESULT hr = S_OK;
    BOOL fwOn;

    _ASSERT(fwProfile != NULL);

    // Check to see if the firewall is off.
    hr = WindowsFirewallIsOn(fwProfile, &fwOn);
    if (FAILED(hr))
    {
        PRINTF("WindowsFirewallIsOn failed: 0x%08lx\n", hr);
        goto error;
    }

    // If it is, turn it on.
    if (!fwOn)
    {
        // Turn the firewall on.
        hr = fwProfile->put_FirewallEnabled(VARIANT_TRUE);
        if (FAILED(hr))
        {
            PRINTF("put_FirewallEnabled failed: 0x%08lx\n", hr);
            goto error;
        }

        PRINTF("The firewall is now on.\n");
    }

error:

    return hr;
}


HRESULT WindowsFirewallTurnOff(IN INetFwProfile* fwProfile)
{
    HRESULT hr = S_OK;
    BOOL fwOn;

    _ASSERT(fwProfile != NULL);

    // Check to see if the firewall is on.
    hr = WindowsFirewallIsOn(fwProfile, &fwOn);
    if (FAILED(hr))
    {
        PRINTF("WindowsFirewallIsOn failed: 0x%08lx\n", hr);
        goto error;
    }

    // If it is, turn it off.
    if (fwOn)
    {
        // Turn the firewall off.
        hr = fwProfile->put_FirewallEnabled(VARIANT_FALSE);
        if (FAILED(hr))
        {
            PRINTF("put_FirewallEnabled failed: 0x%08lx\n", hr);
            goto error;
        }

        PRINTF("The firewall is now off.\n");
    }

error:

    return hr;
}


HRESULT WindowsFirewallCheck(IN INetFwProfile* fwProfile)
{
    HRESULT hr = S_OK;
    BOOL fwOn;

    _ASSERT(fwProfile != NULL);

    // Check to see if the firewall is on.
    hr = WindowsFirewallIsOn(fwProfile, &fwOn);
    if (FAILED(hr))
    {
        goto error;
    }

    // For now we'll just check the standard config because domain can be enabled but won't do anything on a standalone
    if ((fwOn) && (bActive != TRUE))
    {
       //MessageBox(NULL, "Enabled\n", "Firewall Running", MB_OK);
       bActive = TRUE;
    }
    else if((!fwOn) && (bActive != FALSE))
    {
        MessageBox(NULL, "Windows Firewall has been Disabled\n", "Firewall Warning!", MB_OK);
        bActive = FALSE;
    }

    return hr;

error:

    bActive = FALSE;
    return hr;
}


HRESULT WindowsFirewallAppIsEnabled(
            IN INetFwProfile* fwProfile,
            IN const wchar_t* fwProcessImageFileName,
            OUT BOOL* fwAppEnabled
            )
{
    HRESULT hr = S_OK;
    BSTR fwBstrProcessImageFileName = NULL;
    VARIANT_BOOL fwEnabled;
    INetFwAuthorizedApplication* fwApp = NULL;
    INetFwAuthorizedApplications* fwApps = NULL;

    _ASSERT(fwProfile != NULL);
    _ASSERT(fwProcessImageFileName != NULL);
    _ASSERT(fwAppEnabled != NULL);

    *fwAppEnabled = FALSE;

    // Retrieve the authorized application collection.
    hr = fwProfile->get_AuthorizedApplications(&fwApps);
    if (FAILED(hr))
    {
        PRINTF("get_AuthorizedApplications failed: 0x%08lx\n", hr);
        goto error;
    }

    // Allocate a BSTR for the process image file name.
    fwBstrProcessImageFileName = SysAllocString(fwProcessImageFileName);
    if (SysStringLen(fwBstrProcessImageFileName) == 0)
    {
        hr = E_OUTOFMEMORY;
        PRINTF("SysAllocString failed: 0x%08lx\n", hr);
        goto error;
    }

    // Attempt to retrieve the authorized application.
    hr = fwApps->Item(fwBstrProcessImageFileName, &fwApp);
    if (SUCCEEDED(hr))
    {
        // Find out if the authorized application is enabled.
        hr = fwApp->get_Enabled(&fwEnabled);
        if (FAILED(hr))
        {
            PRINTF("get_Enabled failed: 0x%08lx\n", hr);
            goto error;
        }

        if (fwEnabled != VARIANT_FALSE)
        {
            // The authorized application is enabled.
            *fwAppEnabled = TRUE;

            PRINTF(
                "Authorized application %lS is enabled in the firewall.\n",
                fwProcessImageFileName
                );
        }
        else
        {
            PRINTF(
                "Authorized application %lS is disabled in the firewall.\n",
                fwProcessImageFileName
                );
        }
    }
    else
    {
        // The authorized application was not in the collection.
        hr = S_OK;

        PRINTF(
            "Authorized application %lS is disabled in the firewall.\n",
            fwProcessImageFileName
            );
    }

error:

    // Free the BSTR.
    SysFreeString(fwBstrProcessImageFileName);

    // Release the authorized application instance.
    if (fwApp != NULL)
    {
        fwApp->Release();
    }

    // Release the authorized application collection.
    if (fwApps != NULL)
    {
        fwApps->Release();
    }

    return hr;
}


HRESULT WindowsFirewallAddApp(
            IN INetFwProfile* fwProfile,
            IN const wchar_t* fwProcessImageFileName,
            IN const wchar_t* fwName
            )
{
    HRESULT hr = S_OK;
    BOOL fwAppEnabled;
    BSTR fwBstrName = NULL;
    BSTR fwBstrProcessImageFileName = NULL;
    INetFwAuthorizedApplication* fwApp = NULL;
    INetFwAuthorizedApplications* fwApps = NULL;

    _ASSERT(fwProfile != NULL);
    _ASSERT(fwProcessImageFileName != NULL);
    _ASSERT(fwName != NULL);

    // First check to see if the application is already authorized.
    hr = WindowsFirewallAppIsEnabled(
            fwProfile,
            fwProcessImageFileName,
            &fwAppEnabled
            );
    if (FAILED(hr))
    {
        PRINTF("WindowsFirewallAppIsEnabled failed: 0x%08lx\n", hr);
        goto error;
    }

    // Only add the application if it isn't already authorized.
    if (!fwAppEnabled)
    {
        // Retrieve the authorized application collection.
        hr = fwProfile->get_AuthorizedApplications(&fwApps);
        if (FAILED(hr))
        {
            PRINTF("get_AuthorizedApplications failed: 0x%08lx\n", hr);
            goto error;
        }

        // Create an instance of an authorized application.
        hr = CoCreateInstance(
                __uuidof(NetFwAuthorizedApplication),
                NULL,
                CLSCTX_INPROC_SERVER,
                __uuidof(INetFwAuthorizedApplication),
                (void**)&fwApp
                );
        if (FAILED(hr))
        {
            PRINTF("CoCreateInstance failed: 0x%08lx\n", hr);
            goto error;
        }

        // Allocate a BSTR for the process image file name.
        fwBstrProcessImageFileName = SysAllocString(fwProcessImageFileName);
        if (SysStringLen(fwBstrProcessImageFileName) == 0)
        {
            hr = E_OUTOFMEMORY;
            PRINTF("SysAllocString failed: 0x%08lx\n", hr);
            goto error;
        }

        // Set the process image file name.
        hr = fwApp->put_ProcessImageFileName(fwBstrProcessImageFileName);
        if (FAILED(hr))
        {
            PRINTF("put_ProcessImageFileName failed: 0x%08lx\n", hr);
            goto error;
        }

        // Allocate a BSTR for the application friendly name.
        fwBstrName = SysAllocString(fwName);
        if (SysStringLen(fwBstrName) == 0)
        {
            hr = E_OUTOFMEMORY;
            PRINTF("SysAllocString failed: 0x%08lx\n", hr);
            goto error;
        }

        // Set the application friendly name.
        hr = fwApp->put_Name(fwBstrName);
        if (FAILED(hr))
        {
            PRINTF("put_Name failed: 0x%08lx\n", hr);
            goto error;
        }

        // Add the application to the collection.
        hr = fwApps->Add(fwApp);
        if (FAILED(hr))
        {
            PRINTF("Add failed: 0x%08lx\n", hr);
            goto error;
        }

        PRINTF(
            "Authorized application %lS is now enabled in the firewall.\n",
            fwProcessImageFileName
            );
    }

error:

    // Free the BSTRs.
    SysFreeString(fwBstrName);
    SysFreeString(fwBstrProcessImageFileName);

    // Release the authorized application instance.
    if (fwApp != NULL)
    {
        fwApp->Release();
    }

    // Release the authorized application collection.
    if (fwApps != NULL)
    {
        fwApps->Release();
    }

    return hr;
}


HRESULT WindowsFirewallPortIsEnabled(
            IN INetFwProfile* fwProfile,
            IN LONG portNumber,
            IN NET_FW_IP_PROTOCOL ipProtocol,
            OUT BOOL* fwPortEnabled
            )
{
    HRESULT hr = S_OK;
    VARIANT_BOOL fwEnabled;
    INetFwOpenPort* fwOpenPort = NULL;
    INetFwOpenPorts* fwOpenPorts = NULL;

    _ASSERT(fwProfile != NULL);
    _ASSERT(fwPortEnabled != NULL);

    *fwPortEnabled = FALSE;

    // Retrieve the globally open ports collection.
    hr = fwProfile->get_GloballyOpenPorts(&fwOpenPorts);
    if (FAILED(hr))
    {
        PRINTF("get_GloballyOpenPorts failed: 0x%08lx\n", hr);
        goto error;
    }

    // Attempt to retrieve the globally open port.
    hr = fwOpenPorts->Item(portNumber, ipProtocol, &fwOpenPort);
    if (SUCCEEDED(hr))
    {
        // Find out if the globally open port is enabled.
        hr = fwOpenPort->get_Enabled(&fwEnabled);
        if (FAILED(hr))
        {
            PRINTF("get_Enabled failed: 0x%08lx\n", hr);
            goto error;
        }

        if (fwEnabled != VARIANT_FALSE)
        {
            // The globally open port is enabled.
            *fwPortEnabled = TRUE;

            PRINTF("Port %ld is open in the firewall.\n", portNumber);
        }
        else
        {
            PRINTF("Port %ld is not open in the firewall.\n", portNumber);
        }
    }
    else
    {
        // The globally open port was not in the collection.
        hr = S_OK;

        PRINTF("Port %ld is not open in the firewall.\n", portNumber);
    }

error:

    // Release the globally open port.
    if (fwOpenPort != NULL)
    {
        fwOpenPort->Release();
    }

    // Release the globally open ports collection.
    if (fwOpenPorts != NULL)
    {
        fwOpenPorts->Release();
    }

    return hr;
}


HRESULT WindowsFirewallPortAdd(
            IN INetFwProfile* fwProfile,
            IN LONG portNumber,
            IN NET_FW_IP_PROTOCOL ipProtocol,
            IN const wchar_t* name
            )
{
    HRESULT hr = S_OK;
    BOOL fwPortEnabled;
    BSTR fwBstrName = NULL;
    INetFwOpenPort* fwOpenPort = NULL;
    INetFwOpenPorts* fwOpenPorts = NULL;

    _ASSERT(fwProfile != NULL);
    _ASSERT(name != NULL);

    // First check to see if the port is already added.
    hr = WindowsFirewallPortIsEnabled(
            fwProfile,
            portNumber,
            ipProtocol,
            &fwPortEnabled
            );
    if (FAILED(hr))
    {
        PRINTF("WindowsFirewallPortIsEnabled failed: 0x%08lx\n", hr);
        goto error;
    }

    // Only add the port if it isn't already added.
    if (!fwPortEnabled)
    {
        // Retrieve the collection of globally open ports.
        hr = fwProfile->get_GloballyOpenPorts(&fwOpenPorts);
        if (FAILED(hr))
        {
            PRINTF("get_GloballyOpenPorts failed: 0x%08lx\n", hr);
            goto error;
        }

        // Create an instance of an open port.
        hr = CoCreateInstance(
                __uuidof(NetFwOpenPort),
                NULL,
                CLSCTX_INPROC_SERVER,
                __uuidof(INetFwOpenPort),
                (void**)&fwOpenPort
                );
        if (FAILED(hr))
        {
            PRINTF("CoCreateInstance failed: 0x%08lx\n", hr);
            goto error;
        }

        // Set the port number.
        hr = fwOpenPort->put_Port(portNumber);
        if (FAILED(hr))
        {
            PRINTF("put_Port failed: 0x%08lx\n", hr);
            goto error;
        }

        // Set the IP protocol.
        hr = fwOpenPort->put_Protocol(ipProtocol);
        if (FAILED(hr))
        {
            PRINTF("put_Protocol failed: 0x%08lx\n", hr);
            goto error;
        }

        // Allocate a BSTR for the friendly name of the port.
        fwBstrName = SysAllocString(name);
        if (SysStringLen(fwBstrName) == 0)
        {
            hr = E_OUTOFMEMORY;
            PRINTF("SysAllocString failed: 0x%08lx\n", hr);
            goto error;
        }

        // Set the friendly name of the port.
        hr = fwOpenPort->put_Name(fwBstrName);
        if (FAILED(hr))
        {
            PRINTF("put_Name failed: 0x%08lx\n", hr);
            goto error;
        }

        // Opens the port and adds it to the collection.
        hr = fwOpenPorts->Add(fwOpenPort);
        if (FAILED(hr))
        {
            PRINTF("Add failed: 0x%08lx\n", hr);
            goto error;
        }

        PRINTF("Port %ld is now open in the firewall.\n", portNumber);
    }

error:

    // Free the BSTR.
    SysFreeString(fwBstrName);

    // Release the open port instance.
    if (fwOpenPort != NULL)
    {
        fwOpenPort->Release();
    }

    // Release the globally open ports collection.
    if (fwOpenPorts != NULL)
    {
        fwOpenPorts->Release();
    }

    return hr;
}


int CheckServices(SC_HANDLE ShdService, SC_HANDLE IPService)
{
    SERVICE_STATUS_PROCESS IPStatus; 
    SERVICE_STATUS_PROCESS ShdStatus; 
    DWORD dwBytesNeeded = 0;

    if (!QueryServiceStatusEx( 
            ShdService,             // handle to service 
            SC_STATUS_PROCESS_INFO, // info level
            (LPBYTE)&ShdStatus,              // address of structure
            sizeof(SERVICE_STATUS_PROCESS), // size of structure
            &dwBytesNeeded ) )              // if buffer too small
    {
        goto error; 
    }
    if (!QueryServiceStatusEx( 
            IPService,             // handle to service 
            SC_STATUS_PROCESS_INFO, // info level
            (LPBYTE)&IPStatus,              // address of structure
            sizeof(SERVICE_STATUS_PROCESS), // size of structure
            &dwBytesNeeded ) )              // if buffer too small
    {
        goto error; 
    }

    if(((IPStatus.dwCurrentState != SERVICE_RUNNING) || (ShdStatus.dwCurrentState != SERVICE_RUNNING)) && (bServiceActive != FALSE))
    {
       bServiceActive = FALSE;
       MessageBox(NULL, "Warning, critical firewall services have been stopped!\n\nPlease restart the SharedAccess service and IpNat Service or\n" \
          "contact your System Administrator immediately.", "Firewall Warning", MB_OK);
    }
    else if(((IPStatus.dwCurrentState == SERVICE_RUNNING) && (ShdStatus.dwCurrentState == SERVICE_RUNNING)) && (bServiceActive != TRUE))
       bServiceActive = TRUE;

    return 0;

error:

    bServiceActive = FALSE;
    return 1;
}


int __cdecl CheckFirewall()
{
    HRESULT hr = S_OK;
    HRESULT comInit = E_FAIL;
    INetFwProfile* fwProfile = NULL;

    // Initialize COM.
    comInit = CoInitializeEx(0, 
       COINIT_DISABLE_OLE1DDE | COINIT_APARTMENTTHREADED);

   // Ignore RPC_E_CHANGED_MODE; this just means that COM has already been
   // initialized with a different mode. Since we don't care what the mode is,
   // we'll just use the existing mode.
   if (comInit != RPC_E_CHANGED_MODE)
   {
        hr = comInit;
        if (FAILED(hr))
        {
            PRINTF("CoInitializeEx failed: 0x%08lx\n", hr);
            goto error;
        }
   }

    // Retrieve the firewall profile currently in effect.
    hr = WindowsFirewallInitialize(&fwProfile);
    if (FAILED(hr))
    {
        PRINTF("WindowsFirewallInitialize failed: 0x%08lx\n", hr);
        goto error;
    }

    // Open the services for checking
    SC_HANDLE schSCManager;

    // Open a handle to the SC Manager database.         
    schSCManager = OpenSCManager( 
         NULL,                    // local machine 
         NULL,                    // ServicesActive database 
         SC_MANAGER_CONNECT);  // full access rights 
         
    if (NULL == schSCManager) 
       goto error;

    SC_HANDLE ShdService;
    ShdService = OpenService( 
        schSCManager,          // SCM database 
        "SharedAccess",          // service name
        SERVICE_QUERY_STATUS); 
 
    if (ShdService == NULL) 
    { 
        goto error; 
    }

    SC_HANDLE IPService;
    IPService = OpenService( 
        schSCManager,          // SCM database 
        "IpNat",          // service name
        SERVICE_QUERY_STATUS); 
 
    if (IPService == NULL) 
    { 
        goto error; 
    }

    while(1)
    {
      hr = WindowsFirewallCheck(fwProfile);
      hr = CheckServices(ShdService, IPService);
      SleepEx(2000, FALSE);
    }

error:

    if(IPService)
       CloseServiceHandle(IPService);
    if(ShdService)
       CloseServiceHandle(ShdService);
    if(schSCManager)
       CloseServiceHandle(schSCManager);
    // Release the firewall profile.
    WindowsFirewallCleanup(fwProfile);

    // Uninitialize COM.
    if (SUCCEEDED(comInit))
    {
        CoUninitialize();
    }

    return 0;
}


int ConfigureFirewall()
{
   STARTUPINFO si;
   PROCESS_INFORMATION pi;

   ZeroMemory( &si, sizeof(si) );
   si.cb = sizeof(si);
   ZeroMemory( &pi, sizeof(pi) );

   if(!CreateProcess(NULL,
      "rundll32.exe shell32.dll,Control_RunDLL firewall.cpl,Windows Firewall",
      NULL,
      NULL,
      FALSE,
      0,
      NULL,
      NULL,
      &si,
      &pi))
      return 1;

   return 0;
}


/* DrawSystray: Begin to draw a SysTray.  Call this function
			first to be given a HDC to draw to.  Then call
            FinishSystray when done.*/
HDC DrawSystray(void)
{
	HDC hdcScreen;/* Will hold the screen HDC */
	HWND hwndScreen=GetDesktopWindow();/* The HWND for the screen */
	int h,w;/* Height and Width of a small icon */

	/* Get the screen HDC to create a compatible bitmap */
	if( (hdcScreen = GetDC( hwndScreen ) ) == NULL )
		return NULL;

	w = GetSystemMetrics(SM_CXSMICON);
	h = GetSystemMetrics(SM_CYSMICON);

	/* Create an off-screen bitmap that the dynamic SysTray can 
	   be drawn into */
	if( (gSystrayBMP = CreateCompatibleBitmap( hdcScreen, w, h )) == NULL )
	{
		ReleaseDC( hwndScreen,hdcScreen );
		return NULL;
	}

	/* Get an HDC to the newly created off-screen bitmap */
	if( (gSystrayHDC = CreateCompatibleDC( hdcScreen )) == NULL )
	{
		ReleaseDC( hwndScreen,hdcScreen );
		DeleteObject(gSystrayBMP);
		return NULL;
	}

	SelectObject(gSystrayHDC,gSystrayBMP);
	ReleaseDC( hwndScreen,hdcScreen );

	return gSystrayHDC;/* Return an HDC to the off-screen bitmap */
}

/* FinishSystray: Called after DrawSystray to complete drawing and
			redisplay the SysTray to the screen.*/
void FinishSystray(void)
{
	HICON icon;/* An icon to be created for the SysTray */
	ICONINFO iconInfo;/* A struct of info on the new Icon */
	HBITMAP maskBitmap;/* A bitmap to hold the icon mask */
	unsigned char *mask;/* A memory structure to hold the icon mask */
	NOTIFYICONDATA nid;/* Used to update the SysTray */
	int h,w;/* Height and width of the SysTray. */

	w = GetSystemMetrics(SM_CXSMICON);
	h = GetSystemMetrics(SM_CYSMICON);

	/* Create a 0 mask */
	mask = (unsigned char*)malloc(h*(w/8));
	memset(mask,0,h*(w/8));

	/* Create a mask bitmap */
	maskBitmap = CreateBitmap(w,h,1,1,mask);
	free(mask);

	/* Create an ICON base on the bitmap just created */
	iconInfo.fIcon = TRUE;
	iconInfo.xHotspot = 0;
	iconInfo.yHotspot = 0;
	iconInfo.hbmMask = maskBitmap;
	iconInfo.hbmColor = gSystrayBMP; 

	//icon = CreateIconIndirect(&iconInfo); 
   if(bActive && bServiceActive)
      icon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON1));     
   else
      icon = LoadIcon(hInst, MAKEINTRESOURCE(IDI_ICON2));
	/* Setup a SysTray based on the newly created icon */
	memset(&nid,0,sizeof(NOTIFYICONDATA));

	nid.cbSize=sizeof(NOTIFYICONDATA);
	nid.hWnd=gHwnd; 
	nid.uID=0;
	nid.uFlags=NIF_ICON|NIF_MESSAGE |NIF_TIP;
	nid.uCallbackMessage=WM_USER;
	nid.hIcon=icon;
	lstrcpyn(nid.szTip,"XPFiremon",sizeof(nid.szTip));
	if(gAlready==TRUE)
		Shell_NotifyIcon(NIM_MODIFY,&nid);
	else Shell_NotifyIcon(NIM_ADD,&nid);
	gAlready=TRUE;
	DeleteObject(gSystrayBMP);
	DeleteDC(gSystrayHDC);
	DeleteObject(maskBitmap);
	DestroyIcon(icon);
}


/* CloseSystray: Called as the program shuts down.  
			removes the SysTray to the screen.*/
void CloseSystray(void)
{
	NOTIFYICONDATA nid;
	nid.cbSize=sizeof(NOTIFYICONDATA);
	nid.hWnd=gHwnd; 
	nid.uID=0;
	nid.uFlags=NIF_MESSAGE;
	nid.uCallbackMessage=WM_USER;
	Shell_NotifyIcon(NIM_DELETE,&nid);

   // Cleanup some variables
   if(fwProfileStatic)
      free(fwProfileStatic);
}

/* This is the Window Proc for the main hidden window, which serves mainly to route events.*/
LRESULT CALLBACK WindowProc(
  HWND hwnd,      /* Handle to window */
  UINT uMsg,      /* Message identifier */
  WPARAM wParam,  /* First message parameter */
  LPARAM lParam   /* Second message parameter */
)
{
	HDC hdc;/* HDC to draw dynamic SysTray to */
	HMENU menu;/* The popup menu for the SysTray */
	POINT pt;/* A point used to track the popup menu */
	WORD wNotifyCode;/* notification code */
	WORD wID;/* Item, control, or accelerator identifier */
	HWND hwndCtl;/* Handle of control */

	switch(uMsg)
	{
	case WM_CREATE:
		SetTimer(hwnd,10,1000,NULL);
		break;

	case WM_TIMER:
		hdc = DrawSystray();
		FinishSystray();
		gCount++;
		if(gCount==100)
			gCount=0;
		break;

	case WM_CLOSE:
		PostQuitMessage(0);
		CloseSystray();
		break;

	case WM_COMMAND:/* Commands come from the popup menu */
		wNotifyCode = HIWORD(wParam); /* Notification code */
		wID = LOWORD(wParam);         /* Item, control, or accelerator identifier */
		hwndCtl = (HWND) lParam;      /* Handle of control */

		switch(wID)
		{
		case 100:
			PostQuitMessage(0);
			CloseSystray();
			break;

		case 101:
         MessageBox(hwnd,"XP Firewall Monitor\nVersion 0.2\nBy: Inferno(inferno@shadowlair.com)","About",MB_OK);
			break;

      case 102:
         MessageBox(hwnd,"XP Firewall Monitor Help\n\nRight click on the XPFiremon icon to control any\n" \
                                                     "Windows firewall features.  To stop or start the\n" \
                                                     "firewall you must have administrative rights.\n" \
                                                     "The icon in the system tray will be green when the\n" \
                                                     "firewall is active and red if it is deactivated.\n" \
                                                     "XPFiremon will also popup a message box alerting\n" \
                                                     "you to the firewall going down.\n", "XPFiremon Help", MB_OK);
         break;

      case 103:
         WindowsFirewallTurnOff(fwProfileStatic);
         break;
         
      case 104:
         WindowsFirewallTurnOn(fwProfileStatic);
         break;

      case 105:
         ConfigureFirewall();
         break;
		}
		break;

	case WM_USER:/* User events are coming from the SysTray */
		switch(lParam)
		{
		case WM_RBUTTONUP:/* Track the right button up event, do a popup */
			menu = CreatePopupMenu();
         AppendMenu(menu,0,105,"Configure");
         AppendMenu(menu,0,104,"Start Windows Firewall");
         AppendMenu(menu,0,103,"Stop Windows Firewall");
         AppendMenu(menu,0,102,"Help");
			AppendMenu(menu,0,101,"About This Program");
			AppendMenu(menu,0,100,"Exit");
			GetCursorPos(&pt);
			TrackPopupMenu(menu,0,pt.x,pt.y,0,gHwnd,NULL);
			DestroyMenu(menu);
			break;
		}
		break;
	}
	return DefWindowProc(
		hwnd,	/* Handle to window */
		uMsg,	/* Message identifier */
		wParam,	/* First message parameter */
		lParam);/* Second message parameter */
}


int WINAPI WinMain(
  HINSTANCE hInstance,	/* Handle to current instance */
  HINSTANCE hPrevInstance,/* Handle to previous instance */
  LPSTR lpCmdLine,      /* Pointer to command line */
  int nCmdShow          /* Show state of window */
)
{
	WNDCLASS cls;
	MSG msg;


    hInst = hInstance;
    cls.style = 0; 
    cls.lpfnWndProc = (WNDPROC) WindowProc; 
    cls.cbClsExtra = 0; 
    cls.cbWndExtra = 0; 
    cls.hInstance = hInstance; 
    cls.hIcon = LoadIcon((HINSTANCE) NULL, 
        IDI_APPLICATION); 
    cls.hCursor = LoadCursor((HINSTANCE) NULL, 
        IDC_ARROW); 
    cls.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH); 
    cls.lpszMenuName =  NULL; 
    cls.lpszClassName = szClassName; 

    if (!RegisterClass(&cls)) 
        return FALSE; 
    if(!CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)CheckFirewall, NULL, 0, NULL))
       return FALSE;

	gHwnd = CreateWindow(
		szClassName,/* Pointer to registered class name */
		"",/* Pointer to window name */
		WS_OVERLAPPEDWINDOW,/* Window style */
		0,/* Horizontal position of window */
		0,/* Vertical position of window */
		0,/* Window width */
		0,/* Window height */
		NULL,/* Handle to parent or owner window */
		NULL,/* Handle to menu or child-window identifier */
		hInstance,/* Handle to application instance */
		NULL);/* Pointer to window-creation data */


    if (!gHwnd) 
        return FALSE; 
 
	while (GetMessage(&msg, (HWND) NULL, 0, 0)) 
    { 
        TranslateMessage(&msg); 
        DispatchMessage(&msg); 
    } 
 

	return msg.wParam; 
}
